@@ -1,15 +1,17 @@
-from .const import DoorStatus, LockInfo, LockState, LockStatus
+from .const import ConnectionInfo, DoorStatus, LockInfo, LockState, LockStatus
 from .lock import Lock
 from .push import PushLock
-from .session import AuthError
+from .session import AuthError, DisconnectedError
 from .util import local_name_to_serial, serial_to_local_name
 
-__version__ = "0.9.5"
+__version__ = "0.17.1"
 
 __all__ = [
     "AuthError",
-    "Lock",
+    "ConnectionInfo",
+    "DisconnectedError",
     "DoorStatus",
+    "Lock",
     "LockInfo",
     "LockState",
     "LockStatus",
