@@ -8,12 +8,13 @@ from typing import Any, TypeVar, cast
 
 from bleak.backends.device import BLEDevice
 from bleak.backends.scanner import AdvertisementData
-from bleak_retry_connector import BleakError
+from bleak_retry_connector import BleakError, BleakNotFoundError
 
 from .const import (
     APPLE_MFR_ID,
     HAP_FIRST_BYTE,
     YALE_MFR_ID,
+    ConnectionInfo,
     DoorStatus,
     LockInfo,
     LockState,
@@ -21,6 +22,7 @@ from .const import (
 )
 from .lock import Lock
 from .session import AuthError, DisconnectedError, ResponseError
+from .util import is_disconnected_error
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -29,11 +31,20 @@ WrapFuncType = TypeVar("WrapFuncType", bound=Callable[..., Any])
 
 DEFAULT_ATTEMPTS = 3
 
-ADV_UPDATE_COALESCE_SECONDS = 10
+# How long to wait before processing an advertisement change
+ADV_UPDATE_COALESCE_SECONDS = 5.50
 
-HK_UPDATE_COALESCE_SECONDS = 1
+# How long to wait before processing the first update
+FIRST_UPDATE_COALESCE_SECONDS = 0.50
 
-UPDATE_IN_PROGRESS_DEFER_SECONDS = 60
+# How long to wait before processing a HomeKit advertisement change
+HK_UPDATE_COALESCE_SECONDS = 2.75
+
+# How long to wait before processing a manual update request
+MANUAL_UPDATE_COALESCE_SECONDS = 0.75
+
+# How long to wait if we get an update storm from the lock
+UPDATE_IN_PROGRESS_DEFER_SECONDS = 29.50
 
 RETRY_EXCEPTIONS = (
     asyncio.TimeoutError,
@@ -76,8 +87,14 @@ def retry_bluetooth_connection_error(func: WrapFuncType) -> WrapFuncType:
                 return await func(self, *args, **kwargs)
             except AuthError:
                 raise
+            except BleakNotFoundError:
+                # The lock cannot be found so there is no
+                # point in retrying.
+                raise
             except RETRY_EXCEPTIONS as err:
                 if attempt == max_attempts:
+                    if is_disconnected_error(err):
+                        raise DisconnectedError(str(err))
                     raise
                 _LOGGER.debug(
                     "%s: %s error calling %s, retrying...",
@@ -96,8 +113,10 @@ class PushLock:
     def __init__(self, local_name: str) -> None:
         """Init the lock watcher."""
         self._local_name = local_name
+        self._name = local_name
         self._lock_info: LockInfo | None = None
         self._lock_state: LockState | None = None
+        self._connection_info: ConnectionInfo | None = None
         self._last_adv_value = -1
         self._last_hk_state = -1
         self._lock_key: str | None = None
@@ -105,11 +124,15 @@ class PushLock:
         self._ble_device: BLEDevice | None = None
         self._operation_lock = asyncio.Lock()
         self._running = False
-        self._callbacks: list[Callable[[LockState, LockInfo], None]] = []
+        self._callbacks: list[
+            Callable[[LockState, LockInfo, ConnectionInfo], None]
+        ] = []
         self._update_task: asyncio.Task | None = None  # type: ignore[type-arg]
         self._debounce_lock = asyncio.Lock()
         self.loop = asyncio._get_running_loop()
         self._cancel_deferred_update: asyncio.TimerHandle | None = None
+        self.last_error: str | None = None
+        self.auth_error = False
 
     @property
     def local_name(self) -> str:
@@ -119,7 +142,7 @@ class PushLock:
     @property
     def name(self) -> str:
         """Get the name of the lock."""
-        return self._local_name
+        return self._name
 
     @property
     def door_status(self) -> DoorStatus:
@@ -141,8 +164,22 @@ class PushLock:
         """Return the current lock info."""
         return self._lock_info
 
+    @property
+    def connection_info(self) -> ConnectionInfo | None:
+        """Return the current connection info."""
+        return self._connection_info
+
+    @property
+    def ble_device(self) -> BLEDevice | None:
+        """Return the current BLEDevice."""
+        return self._ble_device
+
+    def set_name(self, name: str) -> None:
+        """Set the name of the lock."""
+        self._name = name
+
     def register_callback(
-        self, callback: Callable[[LockState, LockInfo], None]
+        self, callback: Callable[[LockState, LockInfo, ConnectionInfo], None]
     ) -> Callable[[], None]:
         """Register a callback to be called when the lock state changes."""
 
@@ -160,13 +197,14 @@ class PushLock:
     def set_ble_device(self, ble_device: BLEDevice) -> None:
         """Set the ble device."""
         self._ble_device = ble_device
+        self._connection_info = ConnectionInfo(ble_device.rssi)
 
     def _get_lock_instance(self) -> Lock:
         """Get the lock instance."""
         assert self._ble_device is not None  # nosec
         assert self._lock_key is not None  # nosec
         assert self._lock_key_index is not None  # nosec
-        return Lock(self._ble_device, self._lock_key, self._lock_key_index)
+        return Lock(self._ble_device, self._lock_key, self._lock_key_index, self._name)
 
     async def _cancel_any_update(self) -> None:
         """Cancel any update task."""
@@ -180,8 +218,8 @@ class PushLock:
             self._update_task.cancel()
             self._update_task = None
 
-    @retry_bluetooth_connection_error
     @operation_lock
+    @retry_bluetooth_connection_error
     async def lock(self) -> None:
         """Lock the lock."""
         _LOGGER.debug("Starting lock")
@@ -198,8 +236,8 @@ class PushLock:
         await self._cancel_any_update()
         _LOGGER.debug("Finished lock")
 
-    @retry_bluetooth_connection_error
     @operation_lock
+    @retry_bluetooth_connection_error
     async def unlock(self) -> None:
         """Unlock the lock."""
         _LOGGER.debug("Starting unlock")
@@ -216,9 +254,19 @@ class PushLock:
         await self._cancel_any_update()
         _LOGGER.debug("Finished unlock")
 
-    @retry_bluetooth_connection_error
+    async def update(self) -> None:
+        """Request that status be updated."""
+        self._schedule_update(MANUAL_UPDATE_COALESCE_SECONDS)
+
+    async def validate(self) -> None:
+        """Validate lock credentials."""
+        _LOGGER.debug("Starting validate")
+        await self._update()
+        _LOGGER.debug("Finished validate")
+
     @operation_lock
-    async def update(self) -> LockState:
+    @retry_bluetooth_connection_error
+    async def _update(self) -> LockState:
         """Update the lock state."""
         lock = self._get_lock_instance()
         try:
@@ -228,7 +276,9 @@ class PushLock:
                 state = await lock.status()
         except asyncio.CancelledError:
             _LOGGER.debug(
-                "%s: In-progress update canceled due to lock operation", self.name
+                "%s: In-progress update canceled due "
+                "to lock operation or setup timeout",
+                self.name,
             )
             raise
         _LOGGER.debug("%s: Updating lock state", self.name)
@@ -238,11 +288,18 @@ class PushLock:
     def _callback_state(self, lock_state: LockState) -> None:
         """Call the callbacks."""
         assert self._lock_info is not None  # nosec
+        assert self._connection_info is not None  # nosec
         self._lock_state = lock_state
-        _LOGGER.debug("%s: New lock state: %s", self.name, self._lock_state)
+        _LOGGER.debug(
+            "%s: New state: %s %s %s",
+            self.name,
+            self._lock_state,
+            self._lock_info,
+            self._connection_info,
+        )
         for callback in self._callbacks:
             try:
-                callback(lock_state, self._lock_info)
+                callback(lock_state, self._lock_info, self._connection_info)
             except Exception:  # pylint: disable=broad-except
                 _LOGGER.exception("%s: Error calling callback", self.name)
 
@@ -259,7 +316,7 @@ class PushLock:
             ad,
         )
         self.set_ble_device(ble_device)
-        next_update = 0
+        next_update = 0.0
         mfr_data = dict(ad.manufacturer_data)
         if APPLE_MFR_ID in mfr_data and mfr_data[APPLE_MFR_ID][0] == HAP_FIRST_BYTE:
             hk_state = get_homekit_state_num(mfr_data[APPLE_MFR_ID])
@@ -269,13 +326,19 @@ class PushLock:
             # if len(mfr_data[APPLE_MFR_ID]) > 20 and YALE_MFR_ID not in mfr_data:
             # mfr_data[YALE_MFR_ID] = mfr_data[APPLE_MFR_ID][20:]
             if hk_state != self._last_hk_state:
-                next_update = HK_UPDATE_COALESCE_SECONDS
+                if self._last_hk_state == -1:
+                    next_update = FIRST_UPDATE_COALESCE_SECONDS
+                else:
+                    next_update = HK_UPDATE_COALESCE_SECONDS
                 self._last_hk_state = hk_state
         if YALE_MFR_ID in mfr_data:
             current_value = mfr_data[YALE_MFR_ID][0]
             if current_value != self._last_adv_value:
                 if not next_update:
-                    next_update = ADV_UPDATE_COALESCE_SECONDS
+                    if self._last_adv_value == -1:
+                        next_update = FIRST_UPDATE_COALESCE_SECONDS
+                    else:
+                        next_update = ADV_UPDATE_COALESCE_SECONDS
                 self._last_adv_value = current_value
         if _LOGGER.isEnabledFor(logging.DEBUG):
             scheduled_update = None
@@ -301,6 +364,7 @@ class PushLock:
         _LOGGER.debug("Waiting for advertisement callbacks for %s", self.name)
         if self._running:
             raise RuntimeError("Already running")
+        self.last_error = "No Bluetooth advertisement received"
         self._running = True
 
         def _cancel() -> None:
@@ -356,23 +420,49 @@ class PushLock:
 
     async def _queue_update(self) -> None:
         """Watch for updates."""
+        _LOGGER.debug("%s: Update queued", self.name)
         async with self._debounce_lock:
+            _LOGGER.debug("%s: Queued update starting", self.name)
             if not self._running:
+                _LOGGER.debug(
+                    "%s: Queued updated ignored because not running", self.name
+                )
                 return
             _LOGGER.debug("%s: Starting update", self.name)
             try:
-                self._update_task = asyncio.create_task(self.update())
+                self.last_error = "Could not connect"
+                self._update_task = asyncio.create_task(self._update())
                 await self._update_task
             except AuthError as ex:
+                self.auth_error = True
+                self.last_error = (
+                    f"Authentication error: key or slot (key index) is incorrect: {ex}"
+                )
                 _LOGGER.error(
-                    "%s: Auth error, key or slot (key index) is incorrect: %s",
+                    "%s: Auth error: key or slot (key index) is incorrect: %s",
+                    self.name,
+                    ex,
+                    exc_info=True,
+                )
+            except ValueError as ex:
+                self.auth_error = True
+                self.last_error = (
+                    "Authentication value error: key or slot "
+                    f"(key index) is incorrect: {ex}"
+                )
+                _LOGGER.error(
+                    "%s: Auth value error: key or slot (key index) is incorrect: %s",
                     self.name,
                     ex,
                     exc_info=True,
                 )
             except asyncio.CancelledError:
                 _LOGGER.debug("%s: In-progress update canceled", self.name)
-            except Exception:  # pylint: disable=broad-except
+            except asyncio.TimeoutError:
+                self.last_error = "Timed out updating"
+                _LOGGER.exception("%s: Timed out updating", self.name)
+            except Exception as ex:  # pylint: disable=broad-except
+                self.last_error = str(ex)
                 _LOGGER.exception("%s: Error updating", self.name)
 
 
